/*
 * Copyright (c) 2022, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)
 * All rights reserved.
 *
 * Processor:                   FM33LE0xxA
 * http:                        http://www.fmdevelopers.com.cn/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. To provide the most up-to-date information, the revision of our documents 
 *    on the World Wide Web will be the most Current. Your printed copy may be  
 *    an earlier revision. To verify you have the latest information avaliable,
 *    refer to: http://www.fmdevelopers.com.cn/.
 *
 * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED 
   ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
   OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE 
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE 
   USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.
*/
#include "user_init.h"
#include "uart.h"
#include "lin.h"

/**
  * @brief  将RCHF配置为系统时钟源
  *         当系统时钟频率超过32MHz时，必须使能预取指令
  *         当系统时钟频率超过8NHz时，访问跨电源接口(CDIF)外设时钟必须分频
  * @param  clock: RCHF振荡频率值
  * @retval void
  */
void ClockInit(uint32_t clock)
{
    FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);
    switch (clock)
    {
    case FL_RCC_RCHF_FREQUENCY_8MHZ:
        FL_RCC_RCHF_WriteTrimValue(RCHF8M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE);
        break;

    case FL_RCC_RCHF_FREQUENCY_16MHZ:
        FL_RCC_RCHF_WriteTrimValue(RCHF16M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE);
        break;

    case FL_RCC_RCHF_FREQUENCY_24MHZ:
        FL_RCC_RCHF_WriteTrimValue(RCHF24M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE);
        break;

    default:
        FL_RCC_RCHF_WriteTrimValue(RCHF8M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE);
        break;
    }

    FL_RCC_RCHF_SetFrequency(clock);
    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_RCHF);
}

/**
  * @brief  系统时钟输出IO配置:
  *         可配置输出为XTLF、RCLP、RCHF和AHB等
  * @param  void
  * @retval void
  */
void FoutInit(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;

    GPIO_InitStruct.pin = FL_GPIO_PIN_11;
    GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    GPIO_InitStruct.remapPin = FL_DISABLE;

    FL_GPIO_Init( GPIOD, &GPIO_InitStruct );
    
    FL_GPIO_SetFOUT0(GPIO, FL_GPIO_FOUT0_SELECT_RCHF_DIV64);
}

#ifndef MFANG

#ifdef __CC_ARM
#pragma import(__use_no_semihosting)
//标准库需要的支持函数
struct __FILE
{
    int handle;
};
FILE __stdout;
#endif

//定义_sys_exit()以避免使用半主机模式
void _sys_exit(int x)
{
    x = x;
}

//重定义fputc函数
int fputc(int ch, FILE *f)
{
    FL_UART_WriteTXBuff(UART5, (uint8_t)ch);
    while (FL_UART_IsActiveFlag_TXBuffEmpty(UART5) != FL_SET);
    return ch;
}

void DebugUartInit(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    FL_UART_InitTypeDef UART_InitStruct = {0};

    //PD0:UART5-RX   PD1:UART5-TX
    GPIO_InitStruct.pin        = FL_GPIO_PIN_0 | FL_GPIO_PIN_1;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_ENABLE;
    GPIO_InitStruct.remapPin = FL_DISABLE;
    FL_GPIO_Init(GPIOD, &GPIO_InitStruct);

    UART_InitStruct.clockSrc = FL_RCC_UART0_CLK_SOURCE_APB1CLK;

    UART_InitStruct.baudRate            = 115200;                                   //波特率
    UART_InitStruct.dataWidth           = FL_UART_DATA_WIDTH_8B;                    //数据位数
    UART_InitStruct.stopBits            = FL_UART_STOP_BIT_WIDTH_1B;                //停止位
    UART_InitStruct.parity              = FL_UART_PARITY_EVEN;                      //奇偶校验
    UART_InitStruct.transferDirection   = FL_UART_DIRECTION_TX_RX;                  //接收-发送使能
    FL_UART_Init(UART5, &UART_InitStruct);
}

/**
  * @brief  LED配置
  * @param  void
  * @retval void
  */
void LedInit(void)
{
    uint8_t count = 5;

    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};

    FL_GPIO_ResetOutputPin(LED0_GPIO, LED0_PIN);
    FL_GPIO_ResetOutputPin(LED1_GPIO, LED1_PIN);
    FL_GPIO_ResetOutputPin(LED2_GPIO, LED2_PIN);
    FL_GPIO_ResetOutputPin(LED3_GPIO, LED3_PIN);    

    GPIO_InitStruct.pin = LED0_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED0_GPIO, &GPIO_InitStruct);
    
    GPIO_InitStruct.pin = LED1_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED1_GPIO, &GPIO_InitStruct);

    GPIO_InitStruct.pin = LED2_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED2_GPIO, &GPIO_InitStruct);

    GPIO_InitStruct.pin = LED3_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED3_GPIO, &GPIO_InitStruct);
    
    FL_GPIO_SetOutputPin(LED_R_GPIO, LED_R_PIN);
    FL_GPIO_SetOutputPin(LED_G_GPIO, LED_G_PIN);
    FL_GPIO_SetOutputPin(LED_B_GPIO, LED_B_PIN);

    GPIO_InitStruct.pin = LED_R_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED_R_GPIO, &GPIO_InitStruct);

    GPIO_InitStruct.pin = LED_G_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED_G_GPIO, &GPIO_InitStruct);    
    
    GPIO_InitStruct.pin = LED_B_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LED_B_GPIO, &GPIO_InitStruct);        

    while (count--)
    {
        LED0_ON();
        FL_DelayMs(100);
        LED0_OFF();
        FL_DelayMs(100);
    }
}
#endif

/**
  * @brief  更改系统主频为RCHF基准的PLL
  * @param  void
  * @retval void
  */
void Clockchange(void)
{
    //PLL配置
    FL_RCC_PLL_SetClockSource(FL_RCC_PLL_CLK_SOURCE_RCHF);
    FL_RCC_PLL_SetPrescaler(FL_RCC_PLL_PSC_DIV8); //8分频为1M
    FL_RCC_PLL_WriteMultiplier(0x17); //倍频24M
    FL_RCC_PLL_Enable();

    while(!FL_RCC_IsActiveFlag_PLLReady());  //等待建立

    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_PLL);
    FL_RCC_SetAHBPrescaler(FL_RCC_AHBCLK_PSC_DIV1);
    FL_RCC_SetAPB1Prescaler(FL_RCC_APB1CLK_PSC_DIV1);
    FL_RCC_SetAPB2Prescaler(FL_RCC_APB2CLK_PSC_DIV1);
}

/**
  * @brief  NVIC中断关闭
  * @param  void
  * @retval void
  */
void Close_And_ClearIRQ(IRQn_Type IRQn)
{
    NVIC_DisableIRQ(IRQn);
    NVIC_ClearPendingIRQ(IRQn);	
}

/**
  * @brief  用户参数配置
  * @param  void
  * @retval void
  */
void UserInit(void)
{
#ifndef MFANG
    #if (MAIN_CLOCK == 16000000)
    ClockInit(FL_RCC_RCHF_FREQUENCY_16MHZ);
    #elif (MAIN_CLOCK == 24000000)
    ClockInit(FL_RCC_RCHF_FREQUENCY_24MHZ);
    #endif
    /* Update System Core Clock */
    SystemCoreClockUpdate();    
//    LedInit();
    DebugUartInit();
#endif
}
