/*
 * Copyright (c) 2022, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)
 * All rights reserved.
 *
 * Processor:                   FM33LE0xxA
 * http:                        http://www.fmdevelopers.com.cn/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. To provide the most up-to-date information, the revision of our documents 
 *    on the World Wide Web will be the most Current. Your printed copy may be  
 *    an earlier revision. To verify you have the latest information avaliable,
 *    refer to: http://www.fmdevelopers.com.cn/.
 *
 * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED 
   ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
   OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE 
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE 
   USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.
*/
#include "main.h"
#include "user_init.h"
#include "lin.h"
#include "uart.h"

#define FRAME_DATA_MAX_LEN          11  /* 帧最大数据长度 */
#define SCHEDULE_FRAME_MAX_NUM      255 /* 进度表最大帧数量 */

#define FRAME_TYPE_UNCONDITIONAL    0   /* 无条件帧 */
#define FRAME_TYPE_EVENT_TRIGGER    1   /* 事件触发帧 */
#define FRAME_TYPE_SPORADIC         2   /* 偶发帧 */
#define FRAME_TYPE_DIAGNOSTIC       3   /* 诊断帧 */
#define FRAME_TYPE_RESERVED         4   /* 保留帧 */
#define FRAME_TYPE_NO_RESPONSE      5   /* 自定义，主机发送无需回应帧 */

/* 帧处理状态机 */
#define FRAME_STATE_IDLE            0   /* 空闲状态 主：等待任务开始 从：固定波特率：等待间隔段 自动波特率：等待间隔段+同步段 */
#define FRAME_STATE_SEND_SYN        1   /* 主机发送SYN(包括间隔段和同步段) */
#define FRAME_STATE_RECV_SYN        2   /* 固定波特率：接收同步段0x55 */
#define FRAME_STATE_RECV_PID        3   /* 接收PID */
#define FRAME_STATE_RECV_DATA       4   /* 接收数据 */
#define FRAME_STATE_SEND            5   /* 串口发送 */
#define FRAME_STATE_SLEEP           6   /* 休眠状态 */

#if (SLEEP_SUPPORT == 1)
extern uint8_t  g_awake_init;
#endif

/**
  * @brief  计算PID
  * @param  id: 输入id
  * @retval 计算后的PID 
  */
uint8_t calculatePID(uint8_t id)
{
    uint8_t P0 = 0, P1 = 0;

    P0 = ((id) ^ (id >> 1) ^ (id >> 2) ^ (id >> 4)) & 0x01;
    P1 = (~((id >> 1) ^ (id >> 3) ^ (id >> 4) ^ (id >> 5))) & 0x01;

    return ((P1 << 7) | (P0 << 6) | id);
}

/**
  * @brief  计算校验和
  * @param  data: 数据指针
  *         len:  数据长度
  * @retval checkSum: 校验和 
  */
uint8_t calculateChecksum(uint8_t *data, uint8_t len)
{
    uint8_t i;
    uint16_t checkSum = 0;

    for (i = 0; i < len; i++)
    {
        checkSum += data[i];
        if (checkSum >= 0x100)
        {
            checkSum -= 0xFF;
        }
    }
    checkSum = (~checkSum) & 0xFF;

    return checkSum;
}

/**
  * @brief  LIN slp初始化
  * @param  void
  * @retval void
  */
void lin_slp_pin_init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};

    LIN_SLP_DISABLE();

    GPIO_InitStruct.pin = LIN_SLP_PIN;
    GPIO_InitStruct.mode = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    FL_GPIO_Init(LIN_SLP_GPIO, &GPIO_InitStruct);
}


/**
  * @brief  切换UART RX 为输入捕获
  * @param  void
  * @retval void
  */
__STATIC_INLINE void  SwitchIO_UART_TO_ATIM(void)
{
    uint32_t temp_reg;
    
/*  关闭RX 设为高阻态 */
    LIN_UART_RX_GPIO->FCR &= LIN_UART_RX_PIN_FCR_MASK_INV;
    LIN_UART_RX_GPIO->INEN &= LIN_UART_RX_PIN_INV;
    
/*  使能捕获 */
    temp_reg = LIN_UART_CAPTURE_GPIO->FCR;/*使能为数字功能*/
    temp_reg &= LIN_UART_CAPTURE_PIN_FCR_MASK_INV;
    temp_reg |= LIN_UART_CAPTURE_PIN_FCR_DIGIT;
    LIN_UART_CAPTURE_GPIO->FCR = temp_reg;
}


/**
  * @brief  切换输入捕获为UART RX
  * @param  void
  * @retval void
  */
__STATIC_INLINE void SwitchIO_ATIM_TO_UART(void)
{   
    uint32_t temp_reg;
    
/*  关闭捕获  设为高阻态 */
    LIN_UART_CAPTURE_GPIO->FCR &= LIN_UART_CAPTURE_PIN_FCR_MASK_INV;
    LIN_UART_CAPTURE_GPIO->INEN &= LIN_UART_CAPTURE_PIN_INV;

/*  使能RX */
    temp_reg = LIN_UART_RX_GPIO->FCR;/*使能为数字功能*/
    temp_reg &= LIN_UART_RX_PIN_FCR_MASK_INV;
    temp_reg |= LIN_UART_RX_PIN_FCR_DIGIT;
    LIN_UART_RX_GPIO->FCR = temp_reg;    
}


#define Min(a, b) (((a)<(b))?(a):(b))
#define lin_calc_max_res_timeout_cnt(bitLength, size) (uint16_t)((14*(1+(size))*((bitLength)/TIMER_1US_LOAD))/TIME_BASE_PERIOD + 3 )

#if (AUTOBAUD == 1)
/* 同步处理步骤：目的是不符合要求尽快结束重新判断同步 */
#define SYN_STEP_WAIT_BREAK_DOWN                0   //等待间隔段下降沿
#define SYN_STEP_WAIT_BREAK_UP                  1   //等待间隔段上升沿，获取间隔段时长，转step 2
#define SYN_STEP_WAIT_BREAK_DELIMITER_DOWN      2   //等待间隔符下降沿，获取间隔符时长，与间隔段比较，符合要求转step 3，否则此沿用作间隔段下降沿，转step 1
#define SYN_STEP_WAIT_START_BIT_UP              3   //等待起始位上升沿，获取起始位长度，与间隔段比较，符合要求转step 4，否则转step 0
#define SYN_STEP_WAIT_BYTE_BIT_UP_DOWN          4   //等待后续位上升沿/下降沿，当前位和起始位长度比较，差距超出15%判定失败，当前下降沿转step 1，否则转step 0；不超出继续，直到收齐0x55

/* 输入捕获中断判断同步相关变量 */
uint32_t    LinSampleValue[20];     /* 记录中断来时，当前 sysclk 计数值，需要芯片支持sysclk时钟源为APB */
uint16_t    LinSampleCounter;       /* 记录GPTIM PWM中断次数 */

uint32_t    Interval_sync;          //同步段时长
uint32_t    BitLength = MAIN_CLOCK/LIN_BAUDRATE;              //BGR
#endif

/* 帧格式 [最大11字节] */
/* data区内容：受保护ID-1字节; 数据段-0-8字节；校验和-1字节 */
typedef struct s_lin_frame_type
{
    uint8_t tx_len;/* 发送长度 */
    uint8_t rx_len;/* 接收长度 */
    uint8_t data[FRAME_DATA_MAX_LEN];
} lin_frame_data_type;

/* 任务处理：等待接收并处理 */
typedef struct s_lin_task_type
{
    uint8_t frame_state;          /* 帧处理状态机，参照 FRAME_STATE_IDLE 定义 */
    #if (AUTOBAUD == 1)
    uint8_t syn_step;             /* 参照 SYN_STEP_WAIT_BREAK */
    #endif

    /* 帧超时计数，接收到同步段后置起，按照8字节数据计算超时 */
    /* 公式：round((1.4x(10+8x10)xTbit)/Tbase_period) + 3，8为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t frame_timeout_cnt;
    /* 响应超时计数，接收到需要响应的ID后置起，按照实际接收长度计算超时 */
    /* 公式：round((1.4x(10+Nx10)xTbit)/Tbase_period) + 3，N为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t res_frame_timeout_cnt;
    /* Lin总线IDLE超时计数，减为0进入SLEEP状态 */
    uint16_t idle_timeout_cnt;
    lin_frame_data_type lin_frame;/* 帧内容 */
} lin_task_type;

lin_task_type g_lin_task;

uint8_t LIN_CMD0_Data[8];
uint8_t LIN_CMD1_Data[8] = {0x12, 0x34, 0x56, 0x78};
uint8_t LIN_CMD2_Data[8] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88};

uint8_t g_lin_send_len;
uint8_t g_lin_send_buf[8];

lin_cmd_packet_t scheduleTable[] =
{
    /* 用户命令码 */
    {LIN_CMD0, RECEIVE,  LIN_CMD0_LENGTH, LIN_CMD0_Data},
    {LIN_CMD1, TRANSMIT, LIN_CMD1_LENGTH, LIN_CMD1_Data},
//    {LIN_CMD2, TRANSMIT, LIN_CMD2_LENGTH, LIN_CMD2_Data},
};

uint8_t lin_SendData(uint8_t *data, uint8_t len)
{
    g_lin_send_len = len;
    memcpy(g_lin_send_buf, data, len);

    return 0;
}

/**
  * @brief  接收超时定时器初始化
  * @param  void
  * @retval void
  */
void lin_timeout_init(void)
{
    FL_BSTIM32_InitTypeDef        timInit;
    
    timInit.prescaler = 0;                                      /* 分频系数1 */
    timInit.clockSource = FL_RCC_BSTIM32_CLK_SOURCE_APB2CLK;     /* 时钟源 */
    timInit.autoReload = TIMER_LOAD_VALUE - 1;                  /* 自动重装载值 */
    timInit.autoReloadState = FL_DISABLE;                       /* 预装载preload禁能 */
    FL_BSTIM32_Init(BSTIM32, &timInit);

    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    NVIC_ClearPendingIRQ(BSTIM_IRQn);
    
    /* NVIC中断配置 */
    NVIC_DisableIRQ(BSTIM_IRQn);
    NVIC_SetPriority(BSTIM_IRQn, LIN_TIMER_IRQ_PRI);
    NVIC_EnableIRQ(BSTIM_IRQn);
    
    FL_BSTIM32_EnableIT_Update(BSTIM32);

    /* 使能定时器 */
    FL_BSTIM32_Enable(BSTIM32);
}

/**
  * @brief  lin idle
  * @param  void
  * @retval void
  */
void lin_goto_idle(void)
{
    
    g_lin_task.lin_frame.tx_len = 0;
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    UARTxOp.RxLen = 0;
    UARTxOp.TxLen = 0;
    
    #if (AUTOBAUD == 1)
    LinSampleCounter = 0;
    g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;       /* 初始化同步步骤 */

    /* 引脚重置为输入捕获 */
    SwitchIO_UART_TO_ATIM();
    /* 关闭UART接收，使能RX引脚输入捕获中断 */
    FL_UART_DisableRX(LIN_UART);
    
    ATIM->ISR = LIN_ATIM_ISR_MASK;
    NVIC_ClearPendingIRQ(ATIM_IRQn);
    FL_ATIM_EnableIT_CC(ATIM, FL_ATIM_CHANNEL_3);/* 使能捕获中断 */
	FL_ATIM_EnableIT_CC(ATIM, FL_ATIM_CHANNEL_4);/* 使能捕获中断 */
    NVIC_EnableIRQ(ATIM_IRQn);
    
    #else
    FL_UART_EnableRX(LIN_UART);                      /* 非自动波特率打开接收 */
    #endif
    
}


/**
  * @brief  lin idle state初始化
  * @param  void
  * @retval void
  */
__STATIC_INLINE void lin_init_idle_state(void)
{        
    g_lin_task.lin_frame.tx_len = 0;
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    UARTxOp.RxLen = 0;
    UARTxOp.TxLen = 0;
    
    #if (AUTOBAUD == 1)
    LinSampleCounter = 0;
    g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;/* 初始化同步步骤 */
    #else
    
    #endif
}

#if (AUTOBAUD == 1)
#define LOAD_VAULE 0x10000

/**
  * @brief  判断两个bit长度是否超过BIT_GAP
  * @param  cmp_value:  比较值
  *         base_value: 基础值
  * @retval 1: 超过BIT_GAP
  *         0: 不超过BIT_GAP
  */
uint8_t check_bit_gap(uint32_t cmp_value, uint32_t base_value)
{
    uint32_t temp;

    if (cmp_value > base_value)
    {
        temp = cmp_value - base_value;
    }
    else
    {
        temp = base_value - cmp_value;
    }

    if (temp > (uint32_t)(base_value * LIN_SYN_BIT_GAP_PERCENTAGE))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

__STATIC_INLINE uint32_t sampleValueSub(uint32_t cur, uint32_t last)
{
    if (cur < last)
    {
        return (LOAD_VAULE + cur - last);
    }
    else
    {
        return (cur - last);
    }
}

/**
  * @brief  输入捕获中断
  * @note   判断同步间隔段和同步段，计算通信速率
  * @param  void
  * @retval void
  */
void ATIM_IRQHandler(void)
{
    volatile uint32_t clock, temp, cur_bit_length;
    static uint32_t t_break_lenth;
    static uint32_t t_bit_length;
    static uint32_t t_BGR;
    uint32_t isr_flag;
        
    //读取当前时刻
    isr_flag = ATIM->ISR;
    ATIM->ISR = ATIM->ISR; 
    
    LED1_TOG();

    if (isr_flag & LIN_ATIM_ISR_MASK)
    {
        //下降沿
        if (isr_flag & LIN_ATIM_ISR_CH3_MASK)
        {
            clock = ATIM->CCR3;
        }
        //上升沿
        if (isr_flag & LIN_ATIM_ISR_CH4_MASK)
        {
            clock = ATIM->CCR4; 
            
            #if (SLEEP_SUPPORT == 1)
            if (g_awake_init)
            {
                if ((g_lin_task.frame_state == FRAME_STATE_IDLE) ||
                    (g_lin_task.frame_state == FRAME_STATE_SLEEP))
                {
                    g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_UP;
                }
            }
            #endif
        }
                
        if ((g_lin_task.frame_state == FRAME_STATE_IDLE) ||
            (g_lin_task.frame_state == FRAME_STATE_SLEEP))
        {
            switch (g_lin_task.syn_step)
            {
                case SYN_STEP_WAIT_BREAK_DOWN://等待间隔段下降沿
                    LinSampleCounter = 0;
                    if (isr_flag & LIN_ATIM_ISR_CH3_MASK)
                    {//下降沿
                        LinSampleValue[LinSampleCounter] = clock;  
                        g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_UP;
                    }
                    break;
                case SYN_STEP_WAIT_BREAK_UP://等待间隔段上升沿
                    LinSampleCounter++;
                    LinSampleValue[LinSampleCounter] = clock;  
                       
                    #if (SLEEP_SUPPORT == 1)
                    if (g_awake_init)
                    {
                        g_awake_init = 0;
                        t_break_lenth = 300 + clock;/*  带滤波唤醒，最多需要6个lsclk才能唤醒
                                                        按照时钟最慢情况补偿，即XTLF停振，使用内部RCLP，全温区-6.5%~4%偏差
                                                        一个lsclk时长=(1000000/32768)*(1 + 6.5%)us=50us
                                                        即使补偿过度，也可以依靠帧格式进行判帧
                                                     */
                    }
                    else
                    #endif
                    {
                        t_break_lenth = sampleValueSub(LinSampleValue[1], LinSampleValue[0]);
                    }
                    
                    if (t_break_lenth < LIN_MIN_BREAK_VALUE)
                    {
                        g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;
                    }
                    else
                    {
                        g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DELIMITER_DOWN;
                    }
                    break;
                case SYN_STEP_WAIT_BREAK_DELIMITER_DOWN://等待间隔符下降沿
                    LinSampleCounter++;
                    LinSampleValue[LinSampleCounter] = clock;
                    g_lin_task.syn_step = SYN_STEP_WAIT_START_BIT_UP;
                    break;
                case SYN_STEP_WAIT_START_BIT_UP://等待起始位上升沿
                    t_bit_length = sampleValueSub(clock, LinSampleValue[LinSampleCounter]);
                    if (t_break_lenth < t_bit_length * LIN_BREAK_BIT_TIMES)
                    {//间隔段和起始位长度倍数关系，不足则判定为错误
                        if (t_bit_length >= LIN_MIN_BREAK_VALUE)
                        {//作为新的break段
                            t_break_lenth = t_bit_length;
                            LinSampleValue[0] = LinSampleValue[LinSampleCounter];
                            LinSampleValue[1] = clock;
                            LinSampleCounter = 1;
                            g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DELIMITER_DOWN;
                        }
                        else
                        {
                            g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;
                        }
                    }
                    else
                    {
                        LinSampleCounter++;
                        LinSampleValue[LinSampleCounter] = clock;
                        g_lin_task.syn_step = SYN_STEP_WAIT_BYTE_BIT_UP_DOWN;
                    }
                    break;
                case SYN_STEP_WAIT_BYTE_BIT_UP_DOWN://等待后续位上升沿/下降沿
                    cur_bit_length = sampleValueSub(clock, LinSampleValue[LinSampleCounter]);
                    LED1_TOG();//2
                    temp = cur_bit_length;
                    if (temp > t_bit_length)
                    {
                        temp = temp - t_bit_length;
                    }
                    else
                    {
                        temp = t_bit_length - temp;
                    }
                    
                    if (temp > (uint32_t)(t_bit_length * LIN_SYN_BIT_GAP_PERCENTAGE))
                    {//同步段各位与起始位差距，超出则判定为错误，可以快速结束同步判断，重新开始
                        if (isr_flag & LIN_ATIM_ISR_CH4_MASK)
                        {//上升沿
                            if (cur_bit_length >= LIN_MIN_BREAK_VALUE)
                            {//作为新的break段
                                t_break_lenth = cur_bit_length;
                                LinSampleValue[0] = LinSampleValue[LinSampleCounter];
                                LinSampleValue[1] = clock;
                                LinSampleCounter = 1;
                                g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DELIMITER_DOWN;
                            }
                            else
                            {
                                g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;
                            }
                        }
                        else
                        {//下降沿作为间隔段开始
                            LinSampleCounter = 0;
                            LinSampleValue[LinSampleCounter] = clock;  
                            g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_UP;
                        }
                    }
                    else
                    {
                        LED1_TOG();//3
                        LinSampleCounter++;
                        LinSampleValue[LinSampleCounter] = clock;
                        if (LinSampleCounter == 10)
                        {
                            //提前计算波特率，避免最后一位占用太多时间
                            Interval_sync = sampleValueSub(LinSampleValue[10], LinSampleValue[2]); //8位   //第7位下降沿时刻 - 起始位的下降沿时刻
            
                            BitLength = Interval_sync >> 3;//1位时间
                            BitLength = BitLength * LIN_CAPTURE_TIMER_PSC;
                            t_BGR = BitLength - 1;
                        }
                    }
                    
                    break;
                default:
                    break;
            }
        }
         
        if (LinSampleCounter >= 11)
        {//大于等于11次中断，正常应该为到同步段倒数第二位
            LED1_TOG();//4
            
            //关闭输入捕获中断
            ATIM->DIER &= ~LIN_ATIM_IER_MASK;//关闭捕获中断
            
            //检测同步成功
            SwitchIO_ATIM_TO_UART();          
            
            //切换任务状态机
            g_lin_task.lin_frame.rx_len = 0;
            g_lin_task.frame_state = FRAME_STATE_RECV_PID;

            //初始化串口接收长度等，使能接收等待接收ID
            UARTxOp.RxLen = 0 ;
            
            LED1_TOG();//5

            //设置波特率
            temp = LIN_UART->CSR;
            temp &= 0xFFFFFFFC;
            LIN_UART->CSR = temp;
            LIN_UART->BGR = t_BGR;
            LIN_UART->ISR = UART_ISR_RXBF_Msk;
            temp |= 0x3;
            LIN_UART->CSR = temp;

            LED1_TOG();//6          
            
            //开始接收，置超时
            g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, 8);
            
            LED1_TOG();//7
            
            LinSampleCounter = 0;
            g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;
        }        
    }
    
    LED1_TOG();
}
#endif

/**
  * @brief  定时，用于判断字节间超时
  * @note   12位波特率，如1200-10ms 19.2KHz-625us
  * @param  void
  * @retval void
  */
void BSTIM_IRQHandler(void)
{
    if (FL_BSTIM32_IsActiveFlag_Update(BSTIM32))
    {
        FL_BSTIM32_ClearFlag_Update(BSTIM32);
    
        switch (g_lin_task.frame_state)
        {
            case FRAME_STATE_IDLE:
                if (0 == g_lin_task.idle_timeout_cnt)
                {
                    g_lin_task.frame_state = FRAME_STATE_SLEEP;
                    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
                }
                else
                {
                    g_lin_task.idle_timeout_cnt--;
                }
                break;
            case FRAME_STATE_RECV_SYN:
            case FRAME_STATE_RECV_PID:
            case FRAME_STATE_SEND:
                if (0 == g_lin_task.frame_timeout_cnt)
                {
                    //LED2_TOG();
                    
                    lin_goto_idle();
                }
                else
                {
                    g_lin_task.frame_timeout_cnt--;
                }
                break;
            case FRAME_STATE_RECV_DATA:
                if (0 == g_lin_task.res_frame_timeout_cnt)
                {
                    
                    if (UARTxOp.RxLen > 1)
                    {
                        UARTxOp.RxLen = 0;
                    }
                    lin_goto_idle();
                }
                else
                {
                    g_lin_task.res_frame_timeout_cnt--;
                }
                break;
        }
        
    }
}
/**
  * @brief  UART-LIN中断
  * @param  void
  * @retval void
  */
void LIN_UART_IRQHandler(void)
{
    uint8_t tmp08;
    static uint8_t PID, i;
    
    
	/* 接收错误中断处理 需要注意帧类型错误会同时有RXBF标志，用于判定同步段 */
    if (LIN_UART->ISR & UART_ISR_FERR_Msk)
    {
        tmp08 = LIN_UART->RXBUF;
        
        if (tmp08 == LIN_BREAK_BYTE)
        {
            /* 检测到间隔段 */
            g_lin_task.frame_state = FRAME_STATE_RECV_SYN;  
            
            /* 收到间隔段之后置起超时，等待在超时内收到同步字节0x55 */
            #if (AUTOBAUD == 1)
            g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, 8);
            #else
            g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
            #endif
        }
        else
        {
            /* 帧错误 */
            
            lin_goto_idle();
        }

        LIN_UART->ISR = UART_ISR_FERR_Msk;
    }
    /* 接收中断处理 */
    else if (LIN_UART->ISR & UART_ISR_RXBF_Msk)
    {
        switch (g_lin_task.frame_state)
        {
            /* 固定波特率接收同步阶段 */
            case FRAME_STATE_RECV_SYN:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
                UARTxOp.RxLen = 0;
            
                #if (SLEEP_SUPPORT == 1)
                if (g_awake_init)
                {
                    /* 重置超时部分 */
                    g_awake_init = 0;
                    g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
                }
                #endif           
            
                if (tmp08 == LIN_SYN_BYTE)
                {
                    /* 间隔段之后的第一个字节是0x55 */
                    g_lin_task.frame_state = FRAME_STATE_RECV_PID;
                }
                else
                {
                    lin_goto_idle();
                }
                break;
            /* 接收PID阶段，判断PID是否有效如有效判定后续行为是发送还是接收 */
            case FRAME_STATE_RECV_PID:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
            
                UARTxOp.RxBuf[UARTxOp.RxLen] = tmp08;
                UARTxOp.RxLen++;
            
            
                for (i = 0; i < TABLE_SIZE; i++)
                {
                    if (scheduleTable[i].pid == tmp08)
                    {
                        PID = tmp08;
                        break;
                    }
                }
                
                
                if (i == TABLE_SIZE)
                {
                    /* 无对应命令，不做处理 */
                    lin_goto_idle();

                    return;
                }
                else
                {
                    if (scheduleTable[i].type == RECEIVE)
                    {
                        /* 主机发送命令，更新待接收长度，转接收数据阶段 */
                        g_lin_task.lin_frame.rx_len = scheduleTable[i].length + 1;
                        g_lin_task.frame_state = FRAME_STATE_RECV_DATA;
                        
                        /* 更新响应超时 */
                        #if (AUTOBAUD == 1)
                        g_lin_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, scheduleTable[i].length);
                        #else
                        g_lin_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, scheduleTable[i].length);
                        #endif
                    }
                    else
                    {
                        /* 主机接收命令，更新待发送长度，转发送数据阶段 */
                        /* 第一时间发送数据，保证负载率 */
                        
                        
                        #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
                            g_lin_task.lin_frame.data[0] = PID;
                            memcpy(&g_lin_task.lin_frame.data[1],  g_lin_send_buf, g_lin_send_len);
                            g_lin_task.lin_frame.data[g_lin_send_len + 1] = calculateChecksum(g_lin_task.lin_frame.data, g_lin_send_len + 1);

                            g_lin_task.lin_frame.tx_len = g_lin_send_len + 1;
                            UARTxOp.TxLen = g_lin_task.lin_frame.tx_len;
                            UARTxOp.TxOpc = 1;
                            UARTxOp.TxBuf = &g_lin_task.lin_frame.data[1];
                        
                            FL_UART_WriteTXBuff(LIN_UART, UARTxOp.TxBuf[0]);
                        #else
                            memcpy(g_lin_task.lin_frame.data, g_lin_send_buf, g_lin_send_len);
                            g_lin_task.lin_frame.data[g_lin_send_len] = calculateChecksum(g_lin_task.lin_frame.data, g_lin_send_len);

                            g_lin_task.lin_frame.tx_len = g_lin_send_len + 1;
                            UARTxOp.TxLen = g_lin_task.lin_frame.tx_len;
                            UARTxOp.TxOpc = 1;
                            UARTxOp.TxBuf = g_lin_task.lin_frame.data;
                            
                            FL_UART_WriteTXBuff(LIN_UART, UARTxOp.TxBuf[0]);
                        #endif
                        g_lin_send_len = 0;
                        g_lin_task.lin_frame.rx_len = 0;
                        g_lin_task.frame_state = FRAME_STATE_SEND;                        
                    }
                }
                
                break;
            /* 接收数据阶段，直到接收完成才处理 */
            case FRAME_STATE_RECV_DATA:
                if (UARTxOp.RxLen >= g_lin_task.lin_frame.rx_len)
                {
                    /* 最后一个字节，第一时间开接收 */
                    #if (AUTOBAUD == 1)

                    /* 引脚重置为ATIM输入 */
                    SwitchIO_UART_TO_ATIM();

                    /* 关闭UART接收，使能RX引脚输入捕获中断 */
                    ATIM->ISR = LIN_ATIM_ISR_MASK;

                    /* 使能捕获中断 */
                    ATIM->DIER = LIN_ATIM_IER_MASK;

                    /* 关闭UART接收 */
                    LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
                    #else
                    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                    #endif
                    
                }
                
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
                
                UARTxOp.RxBuf[UARTxOp.RxLen] = tmp08;
                UARTxOp.RxLen++;
            
                if (UARTxOp.RxLen >= g_lin_task.lin_frame.rx_len + 1)
                {
            /* 主机主动发送，解析数据 */
#if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
            if (calculateChecksum(UARTxOp.RxBuf, UARTxOp.RxLen - 1) == UARTxOp.RxBuf[UARTxOp.RxLen - 1])
            {
                /* 通过校验 */
                if ((UARTxOp.RxBuf[1] == GetNAD()) || (UARTxOp.RxBuf[1] == NAD_BROADCAST))
                {
                    BOOT_RxMessage[BOOT_MsgIndex & 0x01].length = 8;
                    memcpy(BOOT_RxMessage[BOOT_MsgIndex & 0x01].Data, &UARTxOp.RxBuf[1], 8);
                    BOOT_ExecutiveCommand(&BOOT_RxMessage[BOOT_MsgIndex & 0x01]);
                }
            }
#else
            if (calculateChecksum(&UARTxOp.RxBuf[1], UARTxOp.RxLen - 2) == UARTxOp.RxBuf[UARTxOp.RxLen - 1])
            {
                if ((UARTxOp.RxBuf[1] == GetNAD()) || (UARTxOp.RxBuf[1] == NAD_BROADCAST))
                {
                    BOOT_RxMessage[BOOT_MsgIndex & 0x01].length = 8;
                    memcpy(BOOT_RxMessage[BOOT_MsgIndex & 0x01].Data, &UARTxOp.RxBuf[1], 8);
                    BOOT_ExecutiveCommand(&BOOT_RxMessage[BOOT_MsgIndex & 0x01]);
                }
            }
#endif
                        lin_init_idle_state();
                }
                break;
            case FRAME_STATE_SEND:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
            
                if (UARTxOp.TxOpc < UARTxOp.TxLen)
                {
                    if (tmp08 != UARTxOp.TxBuf[UARTxOp.TxOpc - 1])
                    {   
                        /* bit错误 */
                        #if (AUTOBAUD == 1)
                        
                       /* 引脚重置为ATIM输入 */
                        SwitchIO_UART_TO_ATIM();
                        /* 关闭UART接收，使能RX引脚输入捕获中断 */
                        ATIM->ISR = LIN_ATIM_ISR_MASK;
                        ATIM->DIER = LIN_ATIM_IER_MASK;/* 使能捕获中断 */
                        

                        LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;/* 关闭UART接收 */
                        #else
                        LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                        #endif
                                      
                        
                        lin_init_idle_state();
                    }
                    else
                    {
                        LIN_UART->TXBUF = UARTxOp.TxBuf[UARTxOp.TxOpc]; /* 发送一个数据 */
                        UARTxOp.TxOpc++;
                    }
                }
                else
                {
                    /* 发送完毕，快速切换至接收 */                    
                    #if (AUTOBAUD == 1)

                    /* 引脚重置为GPTIM输入 */
                    SwitchIO_UART_TO_ATIM();

                    /* 关闭UART接收，使能RX引脚输入捕获中断 */
                    ATIM->ISR = LIN_ATIM_ISR_MASK;

                    /* 使能捕获中断 */
                    ATIM->DIER = LIN_ATIM_IER_MASK;

                    /* 关闭UART接收 */
                    LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
                    #else
                    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                    #endif
                    
                    
                    lin_init_idle_state();
                }
                break;
            default:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
            
                break;
        }
    }    
    
    /* 清除可能的错误中断标志 */
    if (LIN_UART->ISR & UART_ISR_PERR_Msk)
    {
        LIN_UART->ISR = UART_ISR_PERR_Msk;
    }
    if (LIN_UART->ISR & UART_ISR_OERR_Msk)
    {
        LIN_UART->ISR = UART_ISR_OERR_Msk;
    }
}

/**
  * @brief  lin UART初始化
  * @param  void
  * @retval void
  */
void lin_uart_init(void)
{
    /* 初始化uart配置 */
    Uartx_Init(LIN_UART);
    
    #if (SLEEP_SUPPORT == 1)
    FL_GPIO_EnableDigitalFilter(GPIO, LIN_RX_EXTI_LINE);        /* EXTI数字滤波功能 */
    #endif

    /* 中断使能 */
    #if (AUTOBAUD == 1)
    #else
    /* 接收错误中断使能，用于同步判断 */
    FL_UART_EnableIT_RXError(LIN_UART);
    #endif
    /* 接收中断使能 */
    FL_UART_EnableIT_RXBuffFull(LIN_UART);

    FL_UART_EnableTX(LIN_UART);
    #if (AUTOBAUD == 1)
    #else
    FL_UART_EnableRX(LIN_UART);
    #endif
}

/**
  * @brief  lin 任务初始化
  * @param  void
  * @retval void
  */
void lin_task_init(void)
{
    uint8_t i;
    
    /* 状态机相关初始化 */
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;

    /* 帧数据初始化 */
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.lin_frame.tx_len = 0;
    
    /* 根据cmd计算pid */
    for (i = 0; i < TABLE_SIZE; i++)
    {
        scheduleTable[i].pid = calculatePID(scheduleTable[i].cmd);
    }
}

#if (AUTOBAUD == 1 )
/**
  * @brief  ATIM捕获初始化
  * @param  void
  * @retval void
  */
void ATIM_Init(void)
{
    FL_ATIM_InitTypeDef        InitStructer1;
    FL_ATIM_IC_InitTypeDef     InitStructer2;
    FL_ATIM_SlaveInitTypeDef   InitStructer3;

    /*-----------------------------------GPIO初始化---------------------------------------*/
    /* 切换IO */
    SwitchIO_UART_TO_ATIM();
    LIN_UART_CAPTURE_GPIO->DFS |= LIN_UART_CAPTURE_PIN;/*使能数字重定向*/
    
    /*-----------------------------时基结构体初始化----------------------------------------*/
    InitStructer1.clockSource           = FL_RCC_ATIM_CLK_SOURCE_APB2CLK;/* 时钟源 APB2 */
    InitStructer1.prescaler             = LIN_CAPTURE_TIMER_PSC - 1;     /* 分频8 */
    InitStructer1.counterMode           = FL_ATIM_COUNTER_DIR_UP;        /* 向上计数 */
    InitStructer1.autoReload            = 65535;                         /* 自动装载值65536 */
    InitStructer1.clockDivision         = FL_ATIM_CLK_DIVISION_DIV1;     /* 死区和数字滤波分频 */
    InitStructer1.repetitionCounter     = 0;                             /* 重复计数0 */
    InitStructer1.autoReloadState       = FL_ENABLE;                     /* 自动重装载ARPE使能 */
    FL_ATIM_Init(ATIM, &InitStructer1);

    /*----------------------------从模式结构体初始化--------------------------------------*/
    InitStructer3.slaveMode     = FL_ATIM_SLAVE_MODE_PROHIBITED;        /* 从机模式禁止 */
    InitStructer3.triggerSrc    = FL_ATIM_TRGI_TI1FP1;                  /* 触发源选择 TI1FP1 */
    InitStructer3.triggerDelay  = FL_DISABLE;                           /* TRGI延迟禁止 */
    FL_ATIM_SlaveMode_Init(ATIM, &InitStructer3);

    /*---------------------------输入捕获结构体初始化-------------------------------------*/
    InitStructer2.ICPolarity    = FL_ATIM_IC_POLARITY_INVERT;           /* 下降沿捕获 */
    InitStructer2.ICActiveInput = FL_ATIM_CHANNEL_MODE_INPUT_NORMAL;    /* CC3配置为输入，IC3映射到TI3 */
    InitStructer2.ICPrescaler   = FL_ATIM_IC_PSC_DIV1;                  /* 输入捕获分频 */
    InitStructer2.ICFilter      = FL_ATIM_IC_FILTER_DIV1;               /* 输入捕获滤波配置 */
    InitStructer2.captureState  = FL_ENABLE;                            /* 使能CC3通道 */
    FL_ATIM_IC_Init(ATIM, FL_ATIM_CHANNEL_3, &InitStructer2);

    InitStructer2.ICPolarity    = FL_ATIM_IC_POLARITY_NORMAL;           /* 上升沿捕获 */
    InitStructer2.ICActiveInput = FL_ATIM_CHANNEL_MODE_INPUT_CROSSOVER; /* CC4配置为输入，IC4映射到TI3 */
    FL_ATIM_IC_Init(ATIM, FL_ATIM_CHANNEL_4, &InitStructer2);
    
    ATIM->ISR = ATIM->ISR;                            /* 清除ATIM相关所有中断标志位 */
    NVIC_ClearPendingIRQ(ATIM_IRQn);
    NVIC_DisableIRQ(ATIM_IRQn);
    NVIC_SetPriority(ATIM_IRQn, LIN_ATIM_IRQ_PRI);    /* 中断优先级配置 */
    NVIC_EnableIRQ(ATIM_IRQn);

    FL_ATIM_ClearFlag_CC(ATIM, FL_ATIM_CHANNEL_3);
    FL_ATIM_EnableIT_CC(ATIM, FL_ATIM_CHANNEL_3);

    FL_ATIM_ClearFlag_CC(ATIM, FL_ATIM_CHANNEL_4);    /* 清除捕获标志 */
    FL_ATIM_EnableIT_CC(ATIM, FL_ATIM_CHANNEL_4);     /* 使能捕获中断 */

    FL_ATIM_Enable(ATIM);                             /* 使能定时器 */
}
#endif   /*(AUTOBAUD == 1 )*/

/**
  * @brief  LIN初始化
  * @param  void
  * @retval void
  */
void lin_init(void)
{
    lin_task_init();
    
    lin_slp_pin_init();
    
    lin_uart_init();
    
    #if (AUTOBAUD == 1)
    
    ATIM_Init();
    
    #else
    
    SwitchIO_ATIM_TO_UART();
    
    #endif
    
    lin_timeout_init();
}

#if (SLEEP_SUPPORT == 1)

/**
  * @brief  LIN休眠前初始化
  * @param  void
  * @retval void
  */
void lin_sleep_init(void)
{
    /* 清除和关闭非唤醒源中断 */
    BSTIM32->IER = 0;
    BSTIM32->ISR = BSTIM32->ISR;
    Close_And_ClearIRQ(BSTIM_IRQn);
 
    /* 关闭UART接收功能 */
    LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
    Close_And_ClearIRQ(LIN_UART_IRQType);
    
    #if (AUTOBAUD == 1)

    /* 禁能定时器 */
    ATIM->CR1 &= ~ATIM_CR1_CEN_Msk;
    /* 提前配置唤醒后状态 */
    ATIM->CNT = 0;
    
    Close_And_ClearIRQ(ATIM_IRQn);
    #else
    
    #endif    
}

/**
  * @brief  LIN唤醒后初始化
  * @param  void
  * @retval void
  */
void lin_awake_init(void)
{
    g_lin_task.lin_frame.tx_len = 0;
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    UARTxOp.RxLen = 0;
    UARTxOp.TxLen = 0;
    
    #if (AUTOBAUD == 1)
    LinSampleCounter = 0;

    /* 初始化同步步骤 */
    g_lin_task.syn_step = SYN_STEP_WAIT_BREAK_DOWN;
    
    g_awake_init = 1;
    NVIC_EnableIRQ(LIN_UART_IRQType);
    #else
    g_awake_init = 1;
    /* 固定波特率唤醒，默认为接收到间隔段，超时改小便于容错 */
    g_lin_task.frame_state = FRAME_STATE_RECV_SYN;  
    /* 收到间隔段之后置起超时，等待在超时内收到同步字节0x55 */
    g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 1);
    #endif
    
    /* 使能非唤醒源中断 */
    BSTIM32->ISR = BSTIM32->ISR;
    BSTIM32->IER |= BSTIM32_IER_UIE_Msk;
    NVIC_EnableIRQ(BSTIM_IRQn);
}

/**
  * @brief  LIN sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t lin_is_sleep(void)
{
    if (g_lin_task.frame_state == FRAME_STATE_SLEEP)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif /* End #if (SLEEP_SUPPORT == 1) */

